@@ -1,4 +1,9 @@
 # License: MIT
+# This file is partially built on SMAC3(https://github.com/automl/SMAC3), which is licensed as follows,
+
+# License: 3-clause BSD
+# Copyright (c) 2016-2018, Ml4AAD Group (http://www.ml4aad.org/)
+# Author: Aaron Klein, Marius Lindauer
 
 import abc
 import logging
@@ -15,6 +20,7 @@ from openbox.utils.config_space import get_one_exchange_neighbourhood, \
 from openbox.acq_maximizer.random_configuration_chooser import ChooserNoCoolDown, ChooserProb
 from openbox.utils.history_container import HistoryContainer, MultiStartHistoryContainer
 from openbox.utils.util_funcs import get_types
+from openbox.utils.constants import MAXINT
 
 
 class AcquisitionFunctionMaximizer(object, metaclass=abc.ABCMeta):
@@ -336,7 +342,7 @@ class LocalSearch(AcquisitionFunctionMaximizer):
             # Get one exchange neighborhood returns an iterator (in contrast of
             # the previously returned list).
             all_neighbors = get_one_exchange_neighbourhood(
-                incumbent, seed=self.rng.seed())
+                incumbent, seed=self.rng.randint(MAXINT))
 
             for neighbor in all_neighbors:
                 s_time = time.time()
@@ -576,7 +582,7 @@ class ScipyOptimizer(AcquisitionFunctionMaximizer):
         super().__init__(acquisition_function, config_space, rng)
         self.random_chooser = ChooserProb(prob=rand_prob, rng=rng)
 
-        types, bounds = get_types(self.config_space)
+        types, bounds = get_types(self.config_space)    # todo: support constant hp in scipy optimizer
         assert all(types == 0), 'Scipy optimizer (L-BFGS-B) only supports Integer and Float parameters.'
         self.bounds = bounds
 
