@@ -21,37 +21,36 @@ class Advisor(object, metaclass=abc.ABCMeta):
     """
 
     def __init__(self, config_space,
-                 task_info,
-                 initial_trials=10,
+                 num_objs=1,
+                 num_constraints=0,
+                 initial_trials=3,
                  initial_configurations=None,
                  init_strategy='random_explore_first',
                  history_bo_data=None,
+                 rand_prob=0.1,
                  optimization_strategy='bo',
-                 surrogate_type=None,
-                 acq_type=None,
-                 acq_optimizer_type='local_random',
+                 surrogate_type='auto',
+                 acq_type='auto',
+                 acq_optimizer_type='auto',
                  ref_point=None,
                  output_dir='logs',
                  task_id='default_task_id',
-                 random_state=None):
+                 random_state=None,
+                 **kwargs):
 
         # Create output (logging) directory.
         # Init logging module.
         # Random seed generator.
-        self.task_info = task_info
-        self.num_objs = task_info['num_objs']
-        self.num_constraints = task_info['num_constraints']
+        self.num_objs = num_objs
+        self.num_constraints = num_constraints
         self.init_strategy = init_strategy
         self.output_dir = output_dir
+        self.task_id = task_id
         self.rng = np.random.RandomState(random_state)
         self.logger = get_logger(self.__class__.__name__)
 
-        history_folder = os.path.join(self.output_dir, 'bo_history')
-        if not os.path.exists(history_folder):
-            os.makedirs(history_folder)
-        self.history_file = os.path.join(history_folder, 'bo_history_%s.json' % task_id)
-
         # Basic components in Advisor.
+        self.rand_prob = rand_prob
         self.optimization_strategy = optimization_strategy
 
         # Init the basic ingredients in Bayesian optimization.
@@ -84,9 +83,89 @@ class Advisor(object, metaclass=abc.ABCMeta):
         self.constraint_models = None
         self.acquisition_function = None
         self.optimizer = None
+        self.auto_alter_model = False
+        self.algo_auto_selection()
         self.check_setup()
         self.setup_bo_basics()
 
+    def algo_auto_selection(self):
+        from ConfigSpace import UniformFloatHyperparameter, UniformIntegerHyperparameter, \
+            CategoricalHyperparameter, OrdinalHyperparameter
+        # analyze config space
+        cont_types = (UniformFloatHyperparameter, UniformIntegerHyperparameter)
+        cat_types = (CategoricalHyperparameter, OrdinalHyperparameter)
+        n_cont_hp, n_cat_hp, n_other_hp = 0, 0, 0
+        for hp in self.config_space.get_hyperparameters():
+            if isinstance(hp, cont_types):
+                n_cont_hp += 1
+            elif isinstance(hp, cat_types):
+                n_cat_hp += 1
+            else:
+                n_other_hp += 1
+        n_total_hp = n_cont_hp + n_cat_hp + n_other_hp
+
+        info_str = ''
+
+        if self.surrogate_type == 'auto':
+            self.auto_alter_model = True
+            if n_total_hp >= 100:
+                self.optimization_strategy = 'random'
+                self.surrogate_type = 'prf'  # for setup procedure
+            elif n_total_hp >= 10:
+                self.surrogate_type = 'prf'
+            elif n_cat_hp > n_cont_hp:
+                self.surrogate_type = 'prf'
+            else:
+                self.surrogate_type = 'gp'
+            info_str += ' surrogate_type: %s.' % self.surrogate_type
+
+        if self.acq_type == 'auto':
+            if self.num_objs == 1:  # single objective
+                if self.num_constraints == 0:
+                    self.acq_type = 'ei'
+                else:   # with constraints
+                    self.acq_type = 'eic'
+            elif self.num_objs <= 4:    # multi objective (<=4)
+                if self.num_constraints == 0:
+                    self.acq_type = 'ehvi'
+                else:   # with constraints
+                    self.acq_type = 'ehvic'
+            else:   # multi objective (>4)
+                if self.num_constraints == 0:
+                    self.acq_type = 'mesmo'
+                else:   # with constraints
+                    self.acq_type = 'mesmoc'
+                self.surrogate_type = 'gp_rbf'
+                info_str = ' surrogate_type: %s.' % self.surrogate_type
+            info_str += ' acq_type: %s.' % self.acq_type
+
+        if self.acq_optimizer_type == 'auto':
+            if n_cat_hp + n_other_hp == 0:  # todo: support constant hp in scipy optimizer
+                self.acq_optimizer_type = 'random_scipy'
+            else:
+                self.acq_optimizer_type = 'local_random'
+            info_str += ' acq_optimizer_type: %s.' % self.acq_optimizer_type
+
+        if info_str != '':
+            info_str = '=== [BO auto selection] ===' + info_str
+            self.logger.info(info_str)
+
+    def alter_model(self, history_container):
+        if not self.auto_alter_model:
+            return
+
+        num_config_evaluated = len(history_container.configurations)
+        num_config_successful = len(history_container.successful_perfs)
+
+        if num_config_evaluated == 300:
+            if self.surrogate_type == 'gp':
+                self.surrogate_type = 'prf'
+                self.logger.info('n_observations=300, change surrogate model from GP to PRF!')
+                if self.acq_optimizer_type == 'random_scipy':
+                    self.acq_optimizer_type = 'local_random'
+                    self.logger.info('n_observations=300, change acq optimizer from random_scipy to local_random!')
+                self.setup_bo_basics()
+
     def check_setup(self):
         """
         Check optimization_strategy, num_objs, num_constraints, acq_type, surrogate_type.
@@ -101,44 +180,22 @@ class Advisor(object, metaclass=abc.ABCMeta):
         # single objective
         if self.num_objs == 1:
             if self.num_constraints == 0:
-                if self.acq_type is None:
-                    self.acq_type = 'ei'
                 assert self.acq_type in ['ei', 'eips', 'logei', 'pi', 'lcb', 'lpei', ]
-                if self.surrogate_type is None:
-                    self.surrogate_type = 'gp'
             else:  # with constraints
-                if self.acq_type is None:
-                    self.acq_type = 'eic'
                 assert self.acq_type in ['eic', ]
-                if self.surrogate_type is None:
-                    self.surrogate_type = 'gp'
                 if self.constraint_surrogate_type is None:
                     self.constraint_surrogate_type = 'gp'
 
         # multi-objective
         else:
             if self.num_constraints == 0:
-                if self.acq_type is None:
-                    self.acq_type = 'ehvi'
                 assert self.acq_type in ['ehvi', 'mesmo', 'usemo', 'parego']
-                if self.surrogate_type is None:
-                    if self.acq_type == 'mesmo':
-                        self.surrogate_type = 'gp_rbf'
-                    else:
-                        self.surrogate_type = 'gp'
                 if self.acq_type == 'mesmo' and self.surrogate_type != 'gp_rbf':
                     self.surrogate_type = 'gp_rbf'
                     self.logger.warning('Surrogate model has changed to Gaussian Process with RBF kernel '
                                         'since MESMO is used. Surrogate_type should be set to \'gp_rbf\'.')
             else:  # with constraints
-                if self.acq_type is None:
-                    self.acq_type = 'ehvic'
                 assert self.acq_type in ['ehvic', 'mesmoc', 'mesmoc2']
-                if self.surrogate_type is None:
-                    if self.acq_type == 'mesmoc':
-                        self.surrogate_type = 'gp_rbf'
-                    else:
-                        self.surrogate_type = 'gp'
                 if self.constraint_surrogate_type is None:
                     if self.acq_type == 'mesmoc':
                         self.constraint_surrogate_type = 'gp_rbf'
@@ -264,6 +321,8 @@ class Advisor(object, metaclass=abc.ABCMeta):
         if history_container is None:
             history_container = self.history_container
 
+        self.alter_model(history_container)
+
         num_config_evaluated = len(history_container.configurations)
         num_config_successful = len(history_container.successful_perfs)
 
@@ -273,6 +332,10 @@ class Advisor(object, metaclass=abc.ABCMeta):
         if self.optimization_strategy == 'random':
             return self.sample_random_configs(1, history_container)[0]
 
+        if (not return_list) and self.rng.random() < self.rand_prob:
+            self.logger.info('Sample random config. rand_prob=%f.' % self.rand_prob)
+            return self.sample_random_configs(1, history_container)[0]
+
         X = convert_configurations_to_array(history_container.configurations)
         Y = history_container.get_transformed_perfs()
         cY = history_container.get_transformed_constraint_perfs()
@@ -331,18 +394,15 @@ class Advisor(object, metaclass=abc.ABCMeta):
             challengers = self.optimizer.maximize(runhistory=history_container,
                                                   num_points=5000)
             if return_list:
+                # Caution: return_list doesn't contain random configs sampled according to rand_prob
                 return challengers.challengers
 
-            is_repeated_config = True
-            repeated_time = 0
-            cur_config = None
-            while is_repeated_config:
-                cur_config = challengers.challengers[repeated_time]  # todo: test small space
-                if cur_config in history_container.configurations:
-                    repeated_time += 1
-                else:
-                    is_repeated_config = False
-            return cur_config
+            for config in challengers.challengers:
+                if config not in history_container.configurations:
+                    return config
+            self.logger.warning('Cannot get non duplicate configuration from BO candidates (len=%d). '
+                                'Sample random config.' % (len(challengers.challengers), ))
+            return self.sample_random_configs(1, history_container)[0]
         else:
             raise ValueError('Unknown optimization strategy: %s.' % self.optimization_strategy)
 
@@ -393,23 +453,28 @@ class Advisor(object, metaclass=abc.ABCMeta):
                 sample_cnt = 0
         return configs
 
-    def save_history(self):
-        """
-        Save the history into a json file.
-        Returns
-        -------
+    def get_history(self):
+        return self.history_container
 
+    def save_history(self, dir_path: str = None, file_name: str = None):
         """
-        self.history_container.save_json(self.history_file)
+        Save history to a json file.
+        """
+        if dir_path is None:
+            dir_path = os.path.join(self.output_dir, 'bo_history')
+        if not os.path.exists(dir_path):
+            os.makedirs(dir_path)
+        if file_name is None:
+            file_name = 'bo_history_%s.json' % self.task_id
+        return self.history_container.save_json(os.path.join(dir_path, file_name))
 
-    def load_history_from_json(self):
+    def load_history_from_json(self, fn=None):
         """
         Load history from a json file.
-        Returns
-        -------
-
         """
-        return self.history_container.load_history_from_json(self.config_space, self.history_file)
+        if fn is None:
+            fn = os.path.join(self.output_dir, 'bo_history', 'bo_history_%s.json' % self.task_id)
+        return self.history_container.load_history_from_json(self.config_space, fn)
 
     def get_suggestions(self):
         raise NotImplementedError
