@@ -11,7 +11,7 @@ from openbox.utils.logging_utils import setup_logger, get_logger
 
 class BOBase(object, metaclass=abc.ABCMeta):
     def __init__(self, objective_function, config_space, task_id='task_id', output_dir='logs/',
-                 random_state=1, initial_runs=3, max_runs=50, runtime_limit=None,
+                 random_state=None, initial_runs=3, max_runs=50, runtime_limit=None,
                  sample_strategy='bo', surrogate_type='gp',
                  history_bo_data: List[OrderedDict] = None,
                  time_limit_per_trial=600):
